#!/bin/sh
#
#  File:        adapted from v7judge.sh
#  Purpose:     used as scripts for contestants to test their programs
#  Author:      Howard Cheng <hchcheng@scg.math.uwaterloo.ca>
#
#  1:  name of file with extension (no path) ie.  A.java
#  2:  name of file with no extension
#  3:  language/compiler name
#
#  It is expected that the name of the file with no extension is the problem
#  letter name, and all input data files are of the form A.*.dat.
#
#  Examples:
#  sh v7judge.sh A.java A java
#  sh v7judge.sh A.c A cpp
#

#
# Time limits
#
CTIMELIMIT=5
JAVATIMELIMIT=20

#
# Name of output files
#
OUTFILE=output.$$
rm -f $OUTFILE

DATAHOME=$HOME

#
# Viewer Command
#
VIEWER="xterm -e less "


export PROB=$4
if [ "$4" == "1" ] ; then
   export PROB=A
fi
if [ "$4" == "2" ] ; then
   export PROB=B
fi
if [ "$4" == "3" ] ; then
   export PROB=C
fi
if [ "$4" == "4" ] ; then
   export PROB=D
fi
if [ "$4" == "5" ] ; then
   export PROB=E
fi
if [ "$4" == "6" ] ; then
   export PROB=F
fi
if [ "$4" == "7" ] ; then
   export PROB=G
fi
if [ "$4" == "8" ] ; then
   export PROB=H
fi
if [ "$4" == "9" ] ; then
   export PROB=I
fi


  for blah in *
  do
     if [ "$blah" != "$1" ] ; then
        if [ "$blah" != "${PROB}-judge" ] ; then
           if [ "$blah" != "jj" ] ; then
              export INFILE=$blah
           fi
        fi
     fi
  done

if [ "$DISPLAY" = "" ] ; then
    echo "Note: DISPLAY environmental variable not set, bummer "
fi

#
# Under pc2 this should never occur, but if hand executed it may
#
if [ "$1" != "" ] ; then
	if [ ! -f "$1" ] ; then
		echo
	    echo "-- source file $1 does not exist   --" >> $OUTFILE
	    echo "Script halted, missing source file --" >> $OUTFILE
	    echo 
	    $VIEWER $OUTFILE
	    exit 2
	fi
fi 

RUN=1

#
# Usage 
#
if [ "$1" = "" ] ; then

    echo "$0 filename basename langname "
    echo 
    echo Compiles filename and runs basename, output to a file named $OUTFILE 
    echo then displays that file using the following command: $VIEWER
    echo 
    echo Parameters:
    echo " filename - name of file with extension (no path) ie.  A.java"
    echo " basename - name of file with no extension ie. A"
    echo " langname - language/compiler name (Ex. java) "
    echo
    echo Example of usage: sh $0 hello.java hello java
    echo
    exit 2

#
# Compile the program
#
elif [ "$3" == "c" ] || [ "$3" == "gcc" ] ; then

    CC=/usr/bin/gcc
    CFLAGS="-O2 -Wall"
    #LIBS="${HOME}/scripts/lockout.o -lm"
    LIBS="-lm"

    # remove target executable
    rm -f $2

    echo "Compiling $1 using $CC "
    echo "======> Compiler listing for $1 using $3 <======" >> $OUTFILE

    ($CC $CFLAGS -o $2 $1 $LIBS 2>&1) >> $OUTFILE

    if [ ! -f ./$2 ] ; then 
      echo "Compile failed - program $2 not created"
      RUN=0
    fi

elif [ "$3" == "c++" ] || [ "$3" == "g++" ] ; then

    CC=/usr/bin/g++
    CFLAGS="-O2 -Wall"
    #LIBS="${HOME}/scripts/lockout.o -lm"
    LIBS="-lm"

    # remove target executable
    rm -f $2

    echo "Compiling $1 using $CC "
    echo "======> Compiler listing for $1 using $3 <======" >> $OUTFILE

    ($CC $CFLAGS -o $2 $1 $LIBS 2>&1) >> $OUTFILE

    if [ ! -f ./$2 ] ; then 
      echo "Compile failed - program $2 not created"
      RUN=0
    fi

elif [ "$3" = "java" ] ; then

    # remove target executable
    rm -f $2.class

    echo "Compiling $1 using javac "
    echo "======> Compiler listing for $1 using $3 <======" >> $OUTFILE

    (javac -O $1 2>&1) >> $OUTFILE

    if [ $? -ne 0 ] ; then 
      echo "Compile failed - program $2.class not created"
      RUN=0
    fi

else

    echo "Error - Unknown language $3, contact Contest Staff "  
    echo "Error - Unknown language $3, contact Contest Staff "  >> $OUTFILE
    RUN=0
fi

# now execute the program on each 
if [ $RUN -eq 1 ] ; then
  rm -f $OUTFILE  
  export TESTFILES="${DATAHOME}/${PROB}.*.dat"
  if [ "$INFILE" != "" ] ; then
     export TESTFILES=$INFILE
  fi
  for testdata in $TESTFILES
  do
    if [ ! -f $testdata ] ; then
      echo "Error: there is no data file of the form ${PROB}.*.dat" >> $OUTFILE
    else
      export STDIN=$testdata
      if [ "$INFILE" != "" ] ; then
         export STDIN="/dev/null"
      fi
      TMPOUT=tmpout.$$
      rm -f $TMPOUT
      echo "-- Running $2 on $testdata (stdin = $STDIN) --"
      echo "======> Output of $2 on $testdata <======" >> $OUTFILE
      if [ "$3" = "java" ] ; then
        bash ${HOME}/scripts/runjava.sh $JAVATIMELIMIT $2 $STDIN $TMPOUT
      else
        ${HOME}/scripts/cwrapper $CTIMELIMIT $2 < $STDIN > $TMPOUT
      fi
      cat $TMPOUT >> $OUTFILE
      echo "======> End of Output on $testdata <======" >> $OUTFILE
    fi
  done
  echo "starting viewer on program's output"
fi

# view results
$VIEWER $OUTFILE

rm -f $OUTFILE $TMPOUT 

#  end of script
